// The robot model.
//
#include <stdio.h>
#include <assert.h>
#include <string.h>

#include "modelerview.h"
#include "modelerapp.h"
#include "modelerdraw.h"
#include "modelerui.h"
#include "mat.h"
#include <FL/gl.h>

// This is a list of the controls for the Robot.
// We'll use these constants to access the values 
// of the controls from the user interface.
enum RobotModelControls
{ 
    XPOS,
	YPOS,
	ZPOS,
	HEIGHT,
	ROBOT_HEAD,
	ROBOT_LEFT_ARM,
	ROBOT_RIGHT_ARM,
	ROBOT_LEFT_FOREARM,
	ROBOT_RIGHT_FOREARM,
	ROBOT_CALVES,
	NUMCONTROLS,
};

// Object part properties.
//
enum ObjectPartProperty
{
	OPP_ROTATION_ANGLE,
	OPP_SCALE,
	OPP_ADJUST_COORD,
	OPP_ADJUST_TRANSLATION,
};


// Size definitions for all robot parts.
//
const double	ROBOT_TORSO_LENGTH_X = 1.;
const double	ROBOT_TORSO_HEIGHT_Y = 2.;
const double	ROBOT_TORSO_WIDTH_Z = 0.5;

const double	ROBOT_HEAD_LENGTH_X = ROBOT_TORSO_LENGTH_X * 0.5;
const double	ROBOT_HEAD_HEIGHT_Y = ROBOT_HEAD_LENGTH_X;
const double	ROBOT_HEAD_WIDTH_Z = ROBOT_TORSO_WIDTH_Z;

const double	ROBOT_EAR_HEIGHT = ROBOT_HEAD_LENGTH_X / 8.;
const double	ROBOT_EAR_R1 = ROBOT_HEAD_LENGTH_X / 4.;
const double	ROBOT_EAR_R2 = ROBOT_EAR_R1;

const double	ROBOT_EYE_RADIUS = ROBOT_EAR_R1 / 3.;

const double	ROBOT_MOUTH_LENGTH_X = ROBOT_HEAD_LENGTH_X / 2.;
const double	ROBOT_MOUTH_HEIGHT_Y = ROBOT_HEAD_HEIGHT_Y / 8.;
const double	ROBOT_MOUTH_WIDTH_Z = ROBOT_EYE_RADIUS / 2.;

const double	ROBOT_ARM_LENGTH_X = ROBOT_TORSO_WIDTH_Z * 2. / 3.;
const double	ROBOT_ARM_HEIGHT_Y = ROBOT_TORSO_HEIGHT_Y / 3.;
const double	ROBOT_ARM_WIDTH_Z = ROBOT_TORSO_WIDTH_Z * 2. / 3.;

const double	ROBOT_FOREARM_HEIGHT = ROBOT_ARM_HEIGHT_Y;
const double	ROBOT_FOREARM_R1 = ROBOT_ARM_LENGTH_X / 2. * 0.8;
const double	ROBOT_FOREARM_R2 = ROBOT_FOREARM_R1;

const double	ROBOT_THIGH_LENGTH_X = ROBOT_TORSO_WIDTH_Z * 2. / 3.;
const double	ROBOT_THIGH_HEIGHT_Y = ROBOT_TORSO_HEIGHT_Y / 3.;
const double	ROBOT_THIGH_WIDTH_Z = ROBOT_TORSO_WIDTH_Z * 2. / 3.;

const double	ROBOT_CALF_HEIGHT = ROBOT_THIGH_HEIGHT_Y;
const double	ROBOT_CALF_R1 = ROBOT_THIGH_LENGTH_X / 2. * 0.8;
const double	ROBOT_CALF_R2 = ROBOT_CALF_R1;

const double	ROBOT_FOOT_LENGTH_X = ROBOT_THIGH_LENGTH_X;
const double	ROBOT_FOOT_HEIGHT_Y = ROBOT_TORSO_HEIGHT_Y / 10.;
const double	ROBOT_FOOT_WIDTH_Z = ROBOT_TORSO_WIDTH_Z * 1.5;

// We'll be getting the instance of the application a lot; 
// might as well have it as a macro.
#define VAL(x) (ModelerApplication::Instance()->GetControlValue(x))

// Modeler controls.
//
ModelerControl controls[NUMCONTROLS];

//-------------------------------------------------------------------

class ObjectPart;

struct TreeNode
{
	TreeNode () {m_pSibling = m_pChild = NULL;}

	ObjectPart	*m_pSibling;
	ObjectPart	*m_pChild;
};


//-------------------------------------------------------------------

class ObjectPart : public TreeNode
{
	public:
		ObjectPart (const char *partName, double x, double y, double z);
		virtual ~ObjectPart () {}

		void SetTranslation (double x, double y, double z)
			{m_translateV = Vec3d (x, y, z); m_doTranslation = TRUE;}
		void SetRotation (double angle, double x, double y, double z)
			{m_rotateV = Vec4d (angle, x, y, z); m_doRotation = TRUE;}
		void SetRotationPoint (double x, double y, double z)
			{m_rotationPointV = Vec3d (x, y, z);}
		void SetScale (double x, double y, double z)
			{m_scaleV = Vec3d (x, y, z); m_doScale = TRUE;}
		void SetAmbientColor(float r, float g, float b)
			{m_ambientColor = Vec3f (r, g, b);}
		void SetDiffuseColor(float r, float g, float b)
			{m_diffuseColor = Vec3f (r, g, b);}

		void SetRotationAngle (double angle)
			{m_rotateV [0] = angle;}
		void AdjustCoord (Vec3d &coordV)
			{m_adjustCoordV = coordV;}
		void AdjustTranslation (Vec3d &trsV)
			{m_adjustTranslationV = trsV;}

		const char* GetName () {return m_pName;}

		virtual void draw () = 0;

	protected:
		void Transform ();

		struct BoxCoord
		{
			double	x;
			double 	y;
			double	z;
		};

		struct CylinderCoord
		{
			double	height;
			double	r1;
			double	r2;
		};

		struct SphereCoord
		{
			double	radius;
		};

		union
		{
			BoxCoord		m_boxCoord;
			CylinderCoord	m_cylinderCoord;
			SphereCoord		m_sphereCoord;
		};

		Vec3d		m_translateV;
		Vec4d		m_rotateV;
		Vec3d		m_rotationPointV;
		Vec3d		m_scaleV;

		Vec3d		m_adjustCoordV;
		Vec3d		m_adjustTranslationV;

		Vec3f		m_ambientColor;
		Vec3f		m_diffuseColor;
		BOOL		m_doTranslation;
		BOOL		m_doRotation;
		BOOL		m_doScale;

	private:
		const char	*m_pName;
};


ObjectPart::ObjectPart (
	const char	*partName,
	double		x,
	double		y,
	double		z)
{
	m_boxCoord.x = x;
	m_boxCoord.y = y;
	m_boxCoord.z = z;
	m_pName = partName;
}


void
ObjectPart::Transform ()
{
	setAmbientColor (m_ambientColor [0], m_ambientColor [1], m_ambientColor [2]);
	setDiffuseColor (m_diffuseColor [0], m_diffuseColor [1], m_diffuseColor [2]);

	if (m_doTranslation)
	{
		glTranslated (m_translateV [0] + m_adjustTranslationV [0],
			m_translateV [1] + m_adjustTranslationV [1],
			m_translateV [2] + m_adjustTranslationV [2]);
	}

	if (m_doScale)
	{
		glScaled (m_scaleV [0], m_scaleV [1], m_scaleV [2]);
	}

	if (m_doRotation)
	{
		glTranslated (m_rotationPointV [0], m_rotationPointV [1], m_rotationPointV [2]);
		glRotated (m_rotateV [0], m_rotateV [1], m_rotateV [2], m_rotateV [3]);
		glTranslated (-m_rotationPointV [0], -m_rotationPointV [1], -m_rotationPointV [2]);
	}
}


//-------------------------------------------------------------------

class Object
{
	public:
		Object () {m_pRootObjectPart = NULL;}
		~Object () {destroyObject (m_pRootObjectPart);}

		void AddObjectPart (const char *pParentPartName, ObjectPart *part);
		void SetObjectPartProperty (const char *pPartName, ObjectPartProperty property, Vec3d &valueV);
		void draw () {drawObject (m_pRootObjectPart);}

	private:
		ObjectPart* FindObjectPart (const char *pPartName, ObjectPart *part);
		void drawObject (ObjectPart *part);
		void destroyObject (ObjectPart *part);

		ObjectPart	*m_pRootObjectPart;
};


void
Object::AddObjectPart (
	const char	*pParentPartName,
	ObjectPart	*part)
{
	ObjectPart	*parent;

	if (pParentPartName != NULL)
	{
		parent = FindObjectPart (pParentPartName, m_pRootObjectPart);
		assert (parent != NULL);

		part->m_pSibling = parent->m_pChild;
		parent->m_pChild = part;
	}
	else
	{
		assert (NULL == m_pRootObjectPart);
		m_pRootObjectPart = part;
	}
}


void
Object::SetObjectPartProperty (
	const char			*pPartName,
	ObjectPartProperty	property,
	Vec3d				&valueV)
{
	ObjectPart	*part;

	part = FindObjectPart (pPartName, m_pRootObjectPart);
	assert (part != NULL);

	switch (property)
	{
		default:
			assert (0);
			break;
		case OPP_ROTATION_ANGLE:
			part->SetRotationAngle (valueV [0]);
			break;
		case OPP_SCALE:
			part->SetScale (valueV [0], valueV [1], valueV [2]);
			break;
		case OPP_ADJUST_COORD:
			part->AdjustCoord (valueV);
			break;
		case OPP_ADJUST_TRANSLATION:
			part->AdjustTranslation (valueV);
			break;
	}
}


ObjectPart*
Object::FindObjectPart (
	const char	*pPartName,
	ObjectPart	*part)
{
	ObjectPart	*foundPart = part;

	if (part != NULL)
	{
		if (strcmp (pPartName, part->GetName ()))
		{
			// This is not the object part we are looking for. Try the
			// child subtree first.
			//
			if (NULL == (foundPart = FindObjectPart (pPartName, part->m_pChild)))
			{
				// None of the child subtrees contains the part we are
				// looking for. Try the sibling subtrees.
				//
				foundPart = FindObjectPart (pPartName, part->m_pSibling);
			}
		}
	}

	return foundPart;
}


void
Object::drawObject (
	ObjectPart	*part)
{
	if (part != NULL)
	{
		glPushMatrix ();
		part->draw ();
		drawObject (part->m_pChild);
		glPopMatrix ();
		drawObject (part->m_pSibling);
	}
}


void
Object::destroyObject (
	ObjectPart *part)
{
	if (part != NULL)
	{
		destroyObject (part->m_pChild);
		destroyObject (part->m_pSibling);
		delete part;
	}
}


//-------------------------------------------------------------------

class BoxPart : public ObjectPart
{
	public:
		BoxPart (const char *pPartName, double x, double y, double z) : ObjectPart (pPartName, x, y, z) {}

		virtual void draw ();
};


void
BoxPart::draw ()
{
	Transform ();
	drawBox (m_boxCoord.x + m_adjustCoordV [0],
		m_boxCoord.y + m_adjustCoordV [1],
		m_boxCoord.z + m_adjustCoordV [2]);
}

//-------------------------------------------------------------------

class CylinderPart : public ObjectPart
{
	public:
		CylinderPart (const char *pPartName, double height, double r1, double r2) : ObjectPart (pPartName, height, r1, r2) {}

		virtual void draw ();
};


void
CylinderPart::draw ()
{
	Transform ();
	drawCylinder (m_cylinderCoord.height + m_adjustCoordV [0],
		m_cylinderCoord.r1 + m_adjustCoordV [1],
		m_cylinderCoord.r2 + m_adjustCoordV [2]);
}

//-------------------------------------------------------------------

class SpherePart : public ObjectPart
{
	public:
		SpherePart (const char *pPartName, double radius) : ObjectPart (pPartName, radius, 0, 0) {}

		virtual void draw ();
};


void
SpherePart::draw ()
{
	Transform ();
	drawSphere (m_sphereCoord.radius + m_adjustCoordV [0]);
}

//-------------------------------------------------------------------

// To make a BoxModel, we inherit off of ModelerView
class RobotModel : public ModelerView 
{
	public:
		// Constructor for the model.  In your model, 
		// make sure you call the ModelerView constructor,
		// as done below.
		RobotModel (int x, int y, int w, int h, char *label) 
			: ModelerView (x,y,w,h,label) {m_pRobotObject = NULL;}
		~RobotModel () {if (m_pRobotObject) delete m_pRobotObject;}

		void	Initialize ();
		virtual void draw ();

	private:
		void Animate ();

		Object	*m_pRobotObject;
};


void
RobotModel::draw ()
{
    // This call takes care of a lot of the nasty projection 
    // matrix stuff.  Unless you want to fudge directly with the 
	// projection matrix, don't bother with this ...
    ModelerView::draw ();

	if (ModelerUserInterface::m_controlsAnimOnMenu->value () != 0)
	{
		Animate ();
	}

	m_pRobotObject->SetObjectPartProperty ("head", OPP_ROTATION_ANGLE, Vec3d (VAL (ROBOT_HEAD), 0, 0));
	m_pRobotObject->SetObjectPartProperty ("left_arm", OPP_ROTATION_ANGLE, Vec3d (VAL (ROBOT_LEFT_ARM), 0, 0));
	m_pRobotObject->SetObjectPartProperty ("right_arm", OPP_ROTATION_ANGLE, Vec3d (VAL (ROBOT_RIGHT_ARM), 0, 0));
	m_pRobotObject->SetObjectPartProperty ("left_forearm", OPP_SCALE, Vec3d (1., 1. + VAL (ROBOT_LEFT_FOREARM), 1.));
	m_pRobotObject->SetObjectPartProperty ("right_forearm", OPP_SCALE, Vec3d (1., 1. + VAL (ROBOT_RIGHT_FOREARM), 1.));

	m_pRobotObject->SetObjectPartProperty ("left_calf", OPP_ADJUST_COORD, Vec3d (VAL (ROBOT_CALVES), 0, 0));
	m_pRobotObject->SetObjectPartProperty ("right_calf", OPP_ADJUST_COORD, Vec3d (VAL (ROBOT_CALVES), 0, 0));
	m_pRobotObject->SetObjectPartProperty ("left_foot", OPP_ADJUST_TRANSLATION, Vec3d (0, 0, VAL (ROBOT_CALVES)));
	m_pRobotObject->SetObjectPartProperty ("right_foot", OPP_ADJUST_TRANSLATION, Vec3d (0, 0, VAL (ROBOT_CALVES)));

	glPushMatrix ();
	glTranslated (VAL (XPOS), VAL (YPOS) + VAL (ROBOT_CALVES) * VAL (HEIGHT), VAL (ZPOS));
	glScaled (1, VAL (HEIGHT), 1);
	glRotated (45., 0, -1., 0);
	m_pRobotObject->draw ();
	glPopMatrix ();
}


void
RobotModel::Animate ()
{
	static BOOL	head_right = TRUE;
	static BOOL	left_arm_up = TRUE;
	static BOOL	right_arm_up = TRUE;
	static BOOL	right_forearm_forward = TRUE;
	static BOOL	calves_up = TRUE;

	// Head goes right and left.
	//
	if (head_right)
	{
		ModelerApplication::Instance()->SetControlValue(ROBOT_HEAD, 
			VAL (ROBOT_HEAD) + controls [ROBOT_HEAD].m_stepsize / 4.);

		if (VAL (ROBOT_HEAD) >= controls [ROBOT_HEAD].m_maximum)
		{
			head_right = FALSE;
		}
	}
	else
	{
		ModelerApplication::Instance()->SetControlValue(ROBOT_HEAD, 
			VAL (ROBOT_HEAD) - controls [ROBOT_HEAD].m_stepsize / 4.);

		if (VAL (ROBOT_HEAD) <= controls [ROBOT_HEAD].m_minimum)
		{
			head_right = TRUE;
		}
	}

	// Left hand goes up and down.
	//
	if (left_arm_up)
	{
		ModelerApplication::Instance()->SetControlValue(ROBOT_LEFT_ARM, 
			VAL (ROBOT_LEFT_ARM) + controls [ROBOT_LEFT_ARM].m_stepsize);

		if (VAL (ROBOT_LEFT_ARM) >= controls [ROBOT_LEFT_ARM].m_maximum)
		{
			left_arm_up = FALSE;
		}
	}
	else
	{
		ModelerApplication::Instance()->SetControlValue(ROBOT_LEFT_ARM, 
			VAL (ROBOT_LEFT_ARM) - controls [ROBOT_LEFT_ARM].m_stepsize);

		if (VAL (ROBOT_LEFT_ARM) <= controls [ROBOT_LEFT_ARM].m_minimum)
		{
			left_arm_up = TRUE;
		}
	}

	// Right hand goes up and down.
	//
	if (right_arm_up)
	{
		if (VAL (ROBOT_RIGHT_ARM) < controls [ROBOT_RIGHT_ARM].m_maximum / 2.)
		{
			// Moving the hand up.
			//
			ModelerApplication::Instance()->SetControlValue(ROBOT_RIGHT_ARM, 
				VAL (ROBOT_RIGHT_ARM) + controls [ROBOT_RIGHT_ARM].m_stepsize / 2.);

		}
		else
		{
			// Right forearm goes forward backward.
			//
			if (right_forearm_forward)
			{
				if (VAL (ROBOT_RIGHT_FOREARM) < controls [ROBOT_RIGHT_FOREARM].m_maximum)
				{
					ModelerApplication::Instance()->SetControlValue(ROBOT_RIGHT_FOREARM, 
						VAL (ROBOT_RIGHT_FOREARM) + controls [ROBOT_RIGHT_FOREARM].m_stepsize / 2.);
				}
				else
				{
					right_forearm_forward = FALSE;
				}
			}
			else
			{
				if (VAL (ROBOT_RIGHT_FOREARM) > controls [ROBOT_RIGHT_FOREARM].m_minimum)
				{
					ModelerApplication::Instance()->SetControlValue(ROBOT_RIGHT_FOREARM, 
						VAL (ROBOT_RIGHT_FOREARM) - controls [ROBOT_RIGHT_FOREARM].m_stepsize / 2.);
				}
				else
				{
					right_forearm_forward = TRUE;
					right_arm_up = FALSE;
				}

			} // if (right_forearm_forward)

		} //if (VAL (ROBOT_RIGHT_ARM) < controls [ROBOT_RIGHT_ARM]...
	}
	else
	{
		// Right arm is moving down.
		//
		if (VAL (ROBOT_RIGHT_ARM) > controls [ROBOT_RIGHT_ARM].m_minimum)
		{
			ModelerApplication::Instance()->SetControlValue(ROBOT_RIGHT_ARM, 
				VAL (ROBOT_RIGHT_ARM) - controls [ROBOT_RIGHT_ARM].m_stepsize / 2.);
		}
		else
		{
			right_arm_up = TRUE;
		}
	} // if (right_arm_up)

	// Robot goes up and down.
	//
	if (calves_up)
	{
		if (VAL (ROBOT_CALVES) < controls [ROBOT_CALVES].m_maximum)
		{
			ModelerApplication::Instance()->SetControlValue(ROBOT_CALVES, 
				VAL (ROBOT_CALVES) + controls [ROBOT_CALVES].m_stepsize / 4.);
		}
		else
		{
			calves_up = FALSE;
		}
	}
	else
	{
		if (VAL (ROBOT_CALVES) > controls [ROBOT_CALVES].m_minimum)
		{
			ModelerApplication::Instance()->SetControlValue(ROBOT_CALVES, 
				VAL (ROBOT_CALVES) - controls [ROBOT_CALVES].m_stepsize / 4.);
		}
		else
		{
			calves_up = TRUE;
		}
	}
}


void
RobotModel::Initialize ()
{
	BoxPart			*pBox;
	CylinderPart	*pCylinder;
	SpherePart		*pSphere;

	// Create the robot object.
	//
	m_pRobotObject = new Object ();

	// Create robot's torso.
	//
	pBox = new BoxPart ("torso", ROBOT_TORSO_LENGTH_X, ROBOT_TORSO_HEIGHT_Y, ROBOT_TORSO_WIDTH_Z);
	pBox->SetAmbientColor(.1f,.1f,.1f);
	pBox->SetDiffuseColor (1., 0.65f, 0);
	m_pRobotObject->AddObjectPart (NULL, pBox);

	// Create robot's head.
	//
	pBox = new BoxPart ("head", ROBOT_HEAD_LENGTH_X, ROBOT_HEAD_HEIGHT_Y, ROBOT_HEAD_WIDTH_Z);
	pBox->SetTranslation ((ROBOT_TORSO_LENGTH_X - ROBOT_HEAD_LENGTH_X) / 2.,
							ROBOT_TORSO_HEIGHT_Y,
							ROBOT_TORSO_WIDTH_Z / 2.);
	pBox->SetRotationPoint (ROBOT_HEAD_LENGTH_X /2., 0, 0);
	pBox->SetRotation (0, 0, 1., 0);
	pBox->SetAmbientColor(.1f,.1f,.1f);
	pBox->SetDiffuseColor (1., 1., 0.7f);
	m_pRobotObject->AddObjectPart ("torso", pBox);

	// Create robot's left ear.
	//
	pCylinder = new CylinderPart ("left_ear", ROBOT_EAR_HEIGHT, ROBOT_EAR_R1, ROBOT_EAR_R2);
	pCylinder->SetTranslation (ROBOT_HEAD_LENGTH_X,
								ROBOT_HEAD_HEIGHT_Y / 2.,
								ROBOT_HEAD_WIDTH_Z / 2.);
	pCylinder->SetRotation (90., 0, 1., 0);
	pCylinder->SetAmbientColor(.1f,.1f,.1f);
	pCylinder->SetDiffuseColor(0,1,.5f);
	m_pRobotObject->AddObjectPart ("head", pCylinder);

	// Create robot's right ear.
	//
	pCylinder = new CylinderPart ("right_ear", ROBOT_EAR_HEIGHT, ROBOT_EAR_R1, ROBOT_EAR_R2);
	pCylinder->SetTranslation (0,
								ROBOT_HEAD_HEIGHT_Y / 2.,
								ROBOT_HEAD_WIDTH_Z / 2.);
	pCylinder->SetRotation (90., 0, -1., 0);
	pCylinder->SetDiffuseColor(0,1,.5f);
	m_pRobotObject->AddObjectPart ("head", pCylinder);

	// Create robot's left eye.
	//
	pSphere = new SpherePart ("left_eye", ROBOT_EYE_RADIUS);
	pSphere->SetTranslation (ROBOT_HEAD_LENGTH_X * 3. / 4.,
								ROBOT_HEAD_HEIGHT_Y * 2. / 3.,
								ROBOT_HEAD_WIDTH_Z);
	pCylinder->SetAmbientColor(.1f,.1f,.1f);
	pSphere->SetDiffuseColor (0, 0, 1.);
	m_pRobotObject->AddObjectPart ("head", pSphere);

	// Create robot's right eye.
	//
	pSphere = new SpherePart ("right_eye", ROBOT_EYE_RADIUS);
	pSphere->SetTranslation (ROBOT_HEAD_LENGTH_X / 4.,
								ROBOT_HEAD_HEIGHT_Y * 2. / 3.,
								ROBOT_HEAD_WIDTH_Z);
	pSphere->SetAmbientColor(.1f,.1f,.1f);
	pSphere->SetDiffuseColor (0, 0, 1.);
	m_pRobotObject->AddObjectPart ("head", pSphere);

	// Create robot's mouth.
	//
	pBox = new BoxPart ("mouth", ROBOT_MOUTH_LENGTH_X, ROBOT_MOUTH_HEIGHT_Y, ROBOT_MOUTH_WIDTH_Z);
	pBox->SetTranslation ((ROBOT_HEAD_LENGTH_X - ROBOT_MOUTH_LENGTH_X) / 2.,
							ROBOT_HEAD_HEIGHT_Y / 4.,
							ROBOT_HEAD_WIDTH_Z);
	pBox->SetAmbientColor (.1f,.1f,.1f);
	pBox->SetDiffuseColor (1., 0, 0);
	m_pRobotObject->AddObjectPart ("head", pBox);

	// Create robot's left arm.
	//
	pBox = new BoxPart ("left_arm", ROBOT_ARM_LENGTH_X, ROBOT_ARM_HEIGHT_Y, ROBOT_ARM_WIDTH_Z);
	pBox->SetTranslation (ROBOT_TORSO_LENGTH_X,
							ROBOT_TORSO_HEIGHT_Y * 0.9 - ROBOT_ARM_HEIGHT_Y,
							(ROBOT_TORSO_WIDTH_Z - ROBOT_ARM_WIDTH_Z) / 2.);
	pBox->SetRotationPoint (0, ROBOT_ARM_HEIGHT_Y * 3. / 4., ROBOT_ARM_WIDTH_Z / 2.);
	pBox->SetRotation (0, -1., 0, 0);
	pBox->SetAmbientColor(.1f,.1f,.1f);
	pBox->SetDiffuseColor (1., 0.65f, 0);
	m_pRobotObject->AddObjectPart ("torso", pBox);

	// Create robot's right arm.
	//
	pBox = new BoxPart ("right_arm", ROBOT_ARM_LENGTH_X, ROBOT_ARM_HEIGHT_Y, ROBOT_ARM_WIDTH_Z);
	pBox->SetTranslation (-ROBOT_ARM_LENGTH_X,
							ROBOT_TORSO_HEIGHT_Y * 0.9 - ROBOT_ARM_HEIGHT_Y,
							(ROBOT_TORSO_WIDTH_Z - ROBOT_ARM_WIDTH_Z) / 2.);
	pBox->SetRotationPoint (ROBOT_ARM_LENGTH_X, ROBOT_ARM_HEIGHT_Y * 3. / 4., ROBOT_ARM_WIDTH_Z / 2.);
	pBox->SetRotation (0, -1., 0, 0);
	pBox->SetAmbientColor(.1f,.1f,.1f);
	pBox->SetDiffuseColor (1., 0.65f, 0);
	m_pRobotObject->AddObjectPart ("torso", pBox);

	// Create robot's left forearm.
	//
	pCylinder = new CylinderPart ("left_forearm", ROBOT_FOREARM_HEIGHT, ROBOT_FOREARM_R1, ROBOT_FOREARM_R2);
	pCylinder->SetTranslation (ROBOT_ARM_LENGTH_X / 2., 0, ROBOT_ARM_WIDTH_Z / 2.);
	pCylinder->SetRotation (90., 1., 0, 0);
	pCylinder->SetAmbientColor(.1f,.1f,.1f);
	pCylinder->SetDiffuseColor (1., 1., 0.7f);
	m_pRobotObject->AddObjectPart ("left_arm", pCylinder);

	// Create robot's right forearm.
	//
	pCylinder = new CylinderPart ("right_forearm", ROBOT_FOREARM_HEIGHT, ROBOT_FOREARM_R1, ROBOT_FOREARM_R2);
	pCylinder->SetTranslation (ROBOT_ARM_LENGTH_X / 2., 0, ROBOT_ARM_WIDTH_Z / 2.);
	pCylinder->SetRotation (90., 1., 0, 0);
	pCylinder->SetAmbientColor(.1f,.1f,.1f);
	pCylinder->SetDiffuseColor (1., 1., 0.7f);
	m_pRobotObject->AddObjectPart ("right_arm", pCylinder);

	// Create robot's left thigh.
	//
	pBox = new BoxPart ("left_thigh", ROBOT_THIGH_LENGTH_X, ROBOT_THIGH_HEIGHT_Y, ROBOT_THIGH_WIDTH_Z);
	pBox->SetTranslation (ROBOT_TORSO_LENGTH_X - ROBOT_THIGH_LENGTH_X - (ROBOT_TORSO_WIDTH_Z - ROBOT_THIGH_WIDTH_Z) / 2.,
							-ROBOT_THIGH_HEIGHT_Y,
							(ROBOT_TORSO_WIDTH_Z - ROBOT_THIGH_WIDTH_Z) / 2.);
	pBox->SetAmbientColor(.1f,.1f,.1f);
	pBox->SetDiffuseColor (1., 0.65f, 0);
	m_pRobotObject->AddObjectPart ("torso", pBox);

	// Create robot's right thigh.
	//
	pBox = new BoxPart ("right_thigh", ROBOT_THIGH_LENGTH_X, ROBOT_THIGH_HEIGHT_Y, ROBOT_THIGH_WIDTH_Z);
	pBox->SetTranslation ((ROBOT_TORSO_WIDTH_Z - ROBOT_THIGH_WIDTH_Z) / 2.,
							-ROBOT_THIGH_HEIGHT_Y,
							(ROBOT_TORSO_WIDTH_Z - ROBOT_THIGH_WIDTH_Z) / 2.);
	pBox->SetAmbientColor(.1f,.1f,.1f);
	pBox->SetDiffuseColor (1., 0.65f, 0);
	m_pRobotObject->AddObjectPart ("torso", pBox);

	// Create robot's left calf.
	//
	pCylinder = new CylinderPart ("left_calf", ROBOT_CALF_HEIGHT, ROBOT_CALF_R1, ROBOT_CALF_R2);
	pCylinder->SetTranslation (ROBOT_THIGH_LENGTH_X / 2., 0, ROBOT_THIGH_WIDTH_Z / 2.);
	pCylinder->SetRotation (90., 1., 0, 0);
	pCylinder->SetAmbientColor(.1f,.1f,.1f);
	pCylinder->SetDiffuseColor (1., 1., 0.7f);
	m_pRobotObject->AddObjectPart ("left_thigh", pCylinder);

	// Create robot's right calf.
	//
	pCylinder = new CylinderPart ("right_calf", ROBOT_CALF_HEIGHT, ROBOT_CALF_R1, ROBOT_CALF_R2);
	pCylinder->SetTranslation (ROBOT_THIGH_LENGTH_X / 2., 0, ROBOT_THIGH_WIDTH_Z / 2.);
	pCylinder->SetRotation (90., 1., 0, 0);
	pCylinder->SetAmbientColor(.1f,.1f,.1f);
	pCylinder->SetDiffuseColor (1., 1., 0.7f);
	m_pRobotObject->AddObjectPart ("right_thigh", pCylinder);

	// Create robot's left foot.
	//
	pBox = new BoxPart ("left_foot", ROBOT_FOOT_LENGTH_X, ROBOT_FOOT_WIDTH_Z, ROBOT_FOOT_HEIGHT_Y);
	pBox->SetTranslation (-ROBOT_FOOT_LENGTH_X / 2.,
							-ROBOT_FOOT_LENGTH_X / 2. - ROBOT_CALF_R1,
							ROBOT_CALF_HEIGHT);
	pBox->SetAmbientColor(.1f,.1f,.1f);
	pBox->SetDiffuseColor (0.73f, 0.22f, 0.78f);
	m_pRobotObject->AddObjectPart ("left_calf", pBox);

	// Create robot's right foot.
	//
	pBox = new BoxPart ("right_foot", ROBOT_FOOT_LENGTH_X, ROBOT_FOOT_WIDTH_Z, ROBOT_FOOT_HEIGHT_Y);
	pBox->SetTranslation (-ROBOT_FOOT_LENGTH_X / 2.,
							-ROBOT_FOOT_LENGTH_X / 2. - ROBOT_CALF_R1,
							ROBOT_CALF_HEIGHT);
	pBox->SetAmbientColor(.1f,.1f,.1f);
	pBox->SetDiffuseColor (0.73f, 0.22f, 0.78f);
	m_pRobotObject->AddObjectPart ("right_calf", pBox);
}


//-------------------------------------------------------------------

// We need to make a creator function, mostly because of
// nasty API stuff that we'd rather stay away from.
ModelerView* createRobotModel(int x, int y, int w, int h, char *label)
{ 
    RobotModel	*pRobot;
	
	pRobot = new RobotModel(x,y,w,h,label); 
	pRobot->Initialize ();

	return pRobot;
}

int
main()
{
	// Initialize the controls
	// Constructor is ModelerControl(name, minimumvalue, maximumvalue, 
	//								 stepsize, defaultvalue)
    // You will want to modify this to accommodate your model.
	controls[XPOS]   = ModelerControl("X Position", -5, 5, 0.1f, 0);
	controls[YPOS]   = ModelerControl("Y Position",  0, 5, 0.1f, 0);
	controls[ZPOS]   = ModelerControl("Z Position", -5, 5, 0.1f, 0);
	controls[HEIGHT] = ModelerControl("Height",      1, 5, 0.1f, 1);
	controls[ROBOT_HEAD] = ModelerControl("Head", -45, 45, 1, 0);
	controls[ROBOT_LEFT_ARM] = ModelerControl("Left Arm", 0, 180, 1, 0);
	controls[ROBOT_RIGHT_ARM] = ModelerControl("Right Arm", 0, 180, 1, 0);
	controls[ROBOT_LEFT_FOREARM] = ModelerControl("Left Forearm", 0, ROBOT_ARM_HEIGHT_Y * 5. / 6., 0.01f, 0);
	controls[ROBOT_RIGHT_FOREARM] = ModelerControl("Right Forearm", 0, ROBOT_ARM_HEIGHT_Y * 5. / 6., 0.01f, 0);
	controls[ROBOT_CALVES] = ModelerControl("Calves", 0, ROBOT_THIGH_HEIGHT_Y * 5. / 6., 0.01f, 0);

    // Initialize the modeler application with your model and the
    // appropriate array of controls.
    ModelerApplication::Instance()->Init(&createRobotModel, controls, NUMCONTROLS);

    return ModelerApplication::Instance()->Run();
}

